@@ -621,7 +621,7 @@ class Agent(Insight, Channel, Catalog, Dataset):
         return Insight_Definition(self._env, self._token, self._slug, definition)
 
     def create_channel(self, name, icon='https://storage.googleapis.com/askdata/smartfeed/icons/Channel@2x.png',
-                       visibility='PRIVATE'):
+                       visibility='PRIVATE', autofollow=True):
 
         data = {
             "userId": self.userid,
@@ -630,7 +630,7 @@ class Agent(Insight, Channel, Catalog, Dataset):
             "agentId": self._agentId,
             "visibility": visibility,
             "code": name,
-            "autofollow": True
+            "autofollow": autofollow
         }
 
         s = requests.Session()
@@ -661,7 +661,7 @@ class Agent(Insight, Channel, Catalog, Dataset):
         return r
 
 
-    def __get_user_by_email(self, user_email):
+    def __get_user_by_email(self, user_email, multiple=False):
 
         url = self._base_url_askdata + "/smartbot/share/principals?q=" + user_email
         s = requests.Session()
@@ -671,8 +671,14 @@ class Agent(Insight, Channel, Catalog, Dataset):
         r = s.get(url=url, headers=self._headers)
         r.raise_for_status()
         if (r != None and r.json() != []):
-            user_id = r.json()[0]["id"]
-            return user_id
+            if(multiple):
+                user_ids = []
+                for user in r.json()[0]:
+                    user_ids.append(user["id"])
+                return user_ids
+            else:
+                user_id = r.json()[0]["id"]
+                return user_id
         else:
             print("Please insert valid email")
             return None
@@ -680,8 +686,8 @@ class Agent(Insight, Channel, Catalog, Dataset):
     '''
     role: VIEW/EDIT
     '''
-    def add_user_to_workspace(self, user_email, role="VIEW"):
-        user_id = self.__get_user_by_email(user_email)
+    def add_user_to_workspace(self, user_email, role="VIEW", add_all=False):
+        user_id = self.__get_user_by_email(user_email, add_all)
 
         data = {
             "principals": [
@@ -706,9 +712,9 @@ class Agent(Insight, Channel, Catalog, Dataset):
     '''
     role: FOLLOWER/ADMIN
     '''
-    def add_user_to_channel(self, channel_slug, user_email, role="FOLLOWER"):
+    def add_user_to_channel(self, channel_slug, user_email, role="FOLLOWER", add_all=False):
 
-        user_id = self.__get_user_by_email(user_email)
+        user_id = self.__get_user_by_email(user_email, add_all)
 
         channel_id = self.get_channel(channel_slug)["id"]
         try:
@@ -750,6 +756,66 @@ class Agent(Insight, Channel, Catalog, Dataset):
         else:
             return None
 
+    def create_qc(self, dataset_slug="", fields:list=[], filters=None, sorted_by=None,  dataset_id="", limit=50, offset=0, pivot=[]):
+        """
+        Pivot columns MUST be contained in field list
+        """
+        if(fields==[]):
+            print("SELECT FIELDS")
+            return
+
+        if sorted_by is None:
+            sorted_by = []
+
+        if filters is None:
+            filters = []
+
+        if(dataset_id=="" and dataset_slug!=""):
+            s = requests.Session()
+            s.keep_alive = False
+            retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+            s.mount('https://', HTTPAdapter(max_retries=retries))
+
+            url = self._base_url_askdata + '/smartbot/agents/agentslug/' + self.agent_slug + '/datasetslug/' + dataset_slug
+
+            headers = {"Authorization": "Bearer" + " " + self._token}
+            response = s.get(url=url, headers=headers)
+            response.raise_for_status()
+            r = response.json()
+
+            datasetId = r["datasetId"]
+
+        elif(dataset_id!=""):
+            datasetId=dataset_id
+        else:
+            print("SPECIFY A DATASET ID OR SLUG")
+            return
+
+        body = {
+            "agentId": self.agent_id,
+            "datasetId": datasetId,
+            "fields": fields,
+            "sortedBy": sorted_by,
+            "filters": filters,
+            "limit": limit,
+            "offset": offset,
+            "pivot": pivot
+        }
+
+        url = self.smart_insight_url + "/composed_queries/sdk"
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {"Authorization": "Bearer" + " " + self._token}
+        response = s.post(url=url, json=body, headers=headers)
+        response.raise_for_status()
+
+        return response.json()
+
+
     def get_dataset_slug_from_id(self, dataset_id: str) -> str:
         """
         get dataset slug by the dataset id instantiated with slug
@@ -872,7 +938,7 @@ class Askdata(SignUp):
         # Agent.__init__(self, self, slug=slug)
         return Agent(self, slug=slug, agent_id=agent_id)
 
-    def get_token(self, username, password):
+    def get_token(self):
         return self._token
 
     def normalize_columns(self, df:pd.DataFrame):
@@ -880,6 +946,7 @@ class Askdata(SignUp):
         from askdata import integration as ig
         return ig.normalize_columns(df)
 
+
     def save_dataset(self, df:pd.DataFrame, workspace: str, dataset_slug="", replace=True, dataset_name="", locale="en", skip_import_values=False):
 
         for col in df.columns:
@@ -1016,6 +1083,30 @@ class Askdata(SignUp):
             r = requests.post(url=dataset_url, headers=headers)
             r.raise_for_status()'''
 
+    def get_dataset_as_df(self, workspace:str, dataset_slug:str):
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        url = self.base_url + '/smartbot/agents/agentslug/' + workspace + '/datasetslug/' + dataset_slug
+
+        headers = {"Authorization": "Bearer" + " " + self._token}
+        response = s.get(url=url, headers=headers)
+        response.raise_for_status()
+        r = response.json()
+
+        datasetId = r["datasetId"]
+
+        url = self.base_url + '/smartdataset/data_table/datasets/'+datasetId+'/file'
+
+        headers = {"Authorization": "Bearer" + " " + self._token}
+        response = s.get(url=url, headers=headers)
+        response.raise_for_status()
+        file = open(datasetId+".parquet", 'wb').write(response.content)
+        return pd.read_parquet(datasetId+".parquet")
+
     def delete_agent(self, agent_slug):
         agent = self.agents_dataframe()[self.agents_dataframe()['slug'] == agent_slug.lower()]
         url = self.base_url + "/smartbot/agents/"+ agent.iloc[0]['id']
@@ -1232,19 +1323,20 @@ class Askdata(SignUp):
             })
         return pd.DataFrame(data=columns)
 
-    def query(self, workspace='', dataset='', fields:list=[], filters=None, sorted_by=None, limit=50, offset=0, pivot=[]):
+    def query(self, workspace='', dataset='', fields:list=[], filters=None, sorted_by=None, limit=50, offset=0, pivot=[], get_raw_data=False):
 
         if (workspace == ''):
             if ('/' in dataset):
                 workspace = dataset.split("/")[0]
                 dataset = dataset.split("/")[1]
 
-        agent = self.agent(slug=workspace)
-        return agent.query(dataset, fields, filters, sorted_by, limit=limit, offset=offset, pivot=pivot)
-
-
-    def get(self, query, workspace='', dataset='', lang="en", fields:list=[], filters=None, sorted_by=None, get_raw_data=False):
+        if(get_raw_data):
+            return self.get("", workspace, dataset, fields=fields, filters=filters, sorted_by=sorted_by, limit=limit, offset=offset, pivot=pivot)
+        else:
+            agent = self.agent(slug=workspace)
+            return agent.query(dataset, fields, filters, sorted_by, limit=limit, offset=offset, pivot=pivot)
 
+    def get(self, query, workspace='', dataset='', lang="en", fields:list=[], filters=None, sorted_by=None, limit=50, offset=0, pivot=[], get_raw_data=False):
 
         s = requests.Session()
         s.keep_alive = False
@@ -1262,7 +1354,6 @@ class Askdata(SignUp):
                 response = s.get(url=authentication_url, headers=headers)
                 response.raise_for_status()
                 r = response.json()
-                agent_id = r["agentId"]
             except Exception as e:
                 logging.error("{}".format(e.__str__()))
                 logging.error("{} workspace not found".format(workspace))
@@ -1279,13 +1370,14 @@ class Askdata(SignUp):
                 response = s.get(url=authentication_url, headers=headers)
                 response.raise_for_status()
                 r = response.json()
-                agent_id = r["agentId"]
                 dataset_id = r["datasetId"]
             except Exception as e:
                 logging.error("{}".format(e.__str__()))
                 logging.error("{} workspace not found".format(workspace))
                 return None
 
+        agent = self.agent(workspace)
+        qc = agent.create_qc(dataset, fields, filters, sorted_by, limit, offset, pivot)
 
         url = self.base_url + "/smartbot/agent-slugs/"+workspace+"/v2/ask"
 
@@ -1297,10 +1389,11 @@ class Askdata(SignUp):
         body = {
             "message": query,
             "language": lang.lower(),
-            "userId": self.userid,
-            "fields": fields,
-            "filters": filters,
-            "sorting": sorted_by,
+            "metadata":[
+                {
+                    "qc": qc
+                }
+            ],
             "getRawData": get_raw_data
         }
 
