@@ -5,6 +5,8 @@ import random
 import logging
 import requests
 import pandas as pd
+import os
+import yaml
 
 ops = {
     '<': operator.lt,
@@ -16,6 +18,12 @@ ops = {
 }
 
 
+root_dir = os.path.abspath(os.path.dirname(__file__))
+yaml_path = os.path.join(root_dir, '../askdata/askdata_config/base_url.yaml')
+with open(yaml_path, 'r') as file:
+    url_list = yaml.load(file, Loader=yaml.FullLoader)
+
+
 def get_conditional_phrases(conditions, phrase1, phrase2):
     bool_array = []
     for condition in conditions:
@@ -87,7 +95,7 @@ def text_from_data(df, examples=None, max_tokens=64, fit_items=True):
         retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
         s.mount('https://', HTTPAdapter(max_retries=retries))
 
-        url = "https://api-dev.askdata.com/cfc/nlg"
+        url = url_list['BASE_URL_CFC_DEV'] + "/nlg"
 
         r = s.post(url=url, headers=headers, json=data)
         r.raise_for_status()
